require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xsmbshare',
  dscmeta_resource_friendly_name: 'xSmbShare',
  dscmeta_resource_name: 'MSFT_xSmbShare',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xSmbShare',
  dscmeta_module_version: '2.2.0.0',
  docs: 'The DSC xSmbShare resource type.
         Automatically generated from version 2.2.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: 'Description of the share',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_folderenumerationmode: {
      type: "Optional[Enum['AccessBased', 'Unrestricted']]",
      desc: 'Specifies which files and folders in the new SMB share are visible to users.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of the SMB Share',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sharestate: {
      type: 'Optional[String]',
      desc: 'Specifies the state of the share',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_readaccess: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies which user is granted read permission to access the share.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_special: {
      type: 'Optional[String]',
      desc: 'Specifies if this share is a Special Share. Admin share, default shares, IPC$ share are examples.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Specifies if the share should be added or removed',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_fullaccess: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies which accounts are granted full permission to access the share.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_sharetype: {
      type: 'Optional[String]',
      desc: 'Specifies the type of the share',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_shadowcopy: {
      type: 'Optional[String]',
      desc: 'Specifies if this share is a ShadowCopy',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_encryptdata: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that the share is encrypted.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_changeaccess: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies which user will be granted modify permission to access the share',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'String',
      desc: 'Path to the share',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_concurrentuserlimit: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Specifies the maximum number of concurrently connected users that the new SMB share may accommodate. If this parameter is set to zero (0), then the number of users is unlimited. The default value is zero (0).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_noaccess: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies which accounts are denied access to the share.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_cachingmode: {
      type: "Optional[Enum['None', 'Manual', 'Programs', 'Documents', 'BranchCache']]",
      desc: 'Specifies the caching mode of the offline files for the SMB share.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
